(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/CodGenCF_LR.sml,v
   Comment: intermediate representation (mostly to be used by CodGenCF).
   Release: scope-0-15_final 
  
     Copyright (C) 2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   NOTES:

   All types used in intermediate representation are defined here.
   Data types only. No logics defined here (except from trivialities).

   CodGenCF_LR.sml,v 1.5 2003/07/22 22:00:48 wasowski Exp
*)

structure CodGenCF_LR = struct

(* some interface *)
open LR


val markno :int = 1

(* actions and guards *)

val always_ref :guardref = markno (* must live here bcs depndent on markno *)
val null_actref :int    = markno

(* system in intermediate representation *)

type lrsystem = { 

	name	 :string,	(* system name used for generating file name *)
	fname    :string,	(* name of the source file                   *)
	
	anatomy  :lratom list,	(* or part of hierarchy			     *)
	mconf    :lratom list,	(* configuration vector			     *)
	tenv     :tenv,		(* transitions array			     *)
	init     :action * lratom list,
				(* initial transition (targets,action)       *)
	
	signals  :signals,	(* list of triggering parts of acts          *)
	
	actions  :actions,	(* unique action bodies (ordered)            *)
	guards	 :guards,	(* unique guard bodies (ordered)             *)

	evtypes  :evtypes,	(* Unmodified from Absyn.sml                 *)
	vartypes :vartypes,	(* Unmodified from Absyn.sml		     *)
	venv     :venv,		(* Unmodified from Absyn.sml		     *)
	cenv     :const env,	(* Unmodified from Absyn.sml                 *)

	queuelen :int option,	(* only SOME if queue is used                *)
	statew   :int,		(* number of machines                        *)
	states   :int		(* number of states (NEEDED?)                *)
}	 
 
(* type of data produced by scheduler *)

type scheduling = {

	eenv	  :eenv,	(* map event names to indices		  *)
	tranidx   :tranidx,     (* map events to position in transarr     *)
	st_dict   :int env,	(* map basic states to entries in anatomy *)
	m_dict    :int env	(* map state machines to indexes in mconf *)
	
}

type lrmodel = (lrsystem * lrsizes) list

end
