(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/test/src/Test.sml,v
   Comment: Auxiliary module supporting testing
   Release: scope-0-15_final

     Copyright (C) 2001-2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Test.sml,v 1.17 2003/07/26 20:59:11 wasowski Exp 
*)

structure Test = struct 

(* where to look for scope: *)

val SCOPE = "./scope";

open Process Say Absyn

type aor = unit
type atr = unit

structure ANDStateSet = Interpr.ANDStateSet
type andstateset = Interpr.andstateset

val testno :int ref = ref 0


exception TestFailed

type confspec = string list

fun tstassert true = ()
  | tstassert false = raise TestFailed
      

fun succ2bool f = f = success

val map2 = ListPair.map
val all2 = ListPair.all
val all  = List.all
val exists = List.exists

fun map3 f (x::xs,y::ys,z::zs) = f(x,y,z)::map3 f (xs,ys,zs)
  | map3 f ([],[],[]) = []
  | map3 f _ = Err.impossible "list should have equal length"

(* FIXME: once all tests are ported to use the TestFn functor this function
   should be moved inside the functor and interfaces should be improved *)
(* FIXME: state based printing out indexes is horrible. Once all modules are
   using functor you have the control over it and it can be done
   declaratively *)
  fun test (testfun :'a -> bool) (tstobj :'a) (tstname :string) :bool =
	let
		val sn  = StringCvt.padLeft #"0" 2 (Int.toString (!testno))
		val _   = print ("[" ^ sn ^ "] ")
		val _   = testno := !testno + 1
		val _   = print ("Testing " ^ tstname ^ "...")
		val res = testfun tstobj
		val _   = if res then print "passed\n" else print "FAILED\n"
	in res end

  fun weak_test (v,NONE  ) = true
    | weak_test (v,SOME d) = v = d

  fun weakly_agree (vallst :''a list) (optlst :''a option list) :bool  =
	length vallst = length optlst andalso 
	ListPair.all weak_test (vallst,optlst)

  fun tokenmatch ( "*", _ ) = true
    | tokenmatch ( _, "*" ) = true
    | tokenmatch ( a, b   ) = a = b


  fun idmatch (id1 :string) (id2: string) :bool =	
	let val ts1 = String.tokens (fn c => c = #".") id1
	    val ts2 = String.tokens (fn c => c = #".") id2
	in length ts1 = length ts2 andalso ListPair.all tokenmatch (ts1,ts2) end
	
  fun idlstmatch (lst1 :string list) (lst2 :string list) :bool =
	length lst1 = length lst2 andalso 
	ListPair.all (fn (a,b) => idmatch a b) (lst1,lst2)


  fun conf_agrees (SOME ini :confspec option, computed :andstateset) :bool =
	let val computed1 = ANDStateSet.listItems computed
	    val computed2 = map (Symbol.name o nameand) computed1 
	in 
		List.all (fn s => List.exists (idmatch s) computed2) ini
	end
    | conf_agrees (NONE, computed) = true

(* FIXME: move to functor after all tets use the functor only *)
fun init (testname :string) :unit = 
       say VNorm ("\nTesting " ^ testname ^ "\n\n") 
       before (testno:=0)

val alltrue = List.all (fn x=>x)

fun finalise (testname :string) (res : bool list) :bool =
      if alltrue res then true 
	before say VLow ("\n" ^ testname ^ " tests passed.\n\n")
      else false 
	before say VLow ("\n" ^ testname ^ " tests FAILED.\n\n")
	
      
fun std_testcases () :string list =
	let val instr = TextIO.openIn("test/src/LIST");
	    fun f lst = case TextIO.inputLine instr of
				  "" => lst 
				| s  => f (s::lst)
	    val ret = map (Substring.string o Substring.dropr Char.isSpace 
				o Substring.all) (rev (f []))
	in ret end

(* generates code in test/sandbox. cleans on entry *)

fun generate (fname :string) (preopts :string) (codgen :string) :bool =
	let val _ = system "rm -Rf test/sandbox" 
	in succ2bool (system (SCOPE ^ " " ^ preopts ^ 
		" -dtest/sandbox --externhdr=extern.h " ^ " -o system " ^ codgen 
		^ " -o extern  -cCstubs " ^ " -o main.c  -cCdrv " ^ fname ))
	end

(* compiles code from test/sandbox *)
(* all C files are compiled *)

fun compile (options :string) :bool = 
    succ2bool (system ("sh -c \"cd test/sandbox && cc -pedantic --ansi -I../../ " 
	^ options ^ " main.c system.c extern.c -o system && cd ..\""))

end
