(*
  File   : /home/cvs/cvsroot/projects/visualstate/scope/test/src/tStaticScope.sml,v
  Comment: testing StaticScope module
  Release: scope-0-15_final

     Copyright (C) 2001-2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  
  NOTES:

  The idea is to check for each transition in test cases if proper static
  scope is returned.

  AT THE MOMENT THE TEST IS MANUAL. YOU HAVE TO SEE THE STATIC SCOPE AND
  DECIDE IF IT IS CORRECT.

  tStaticScope.sml,v 1.8 2003/07/15 14:52:07 wasowski Exp 
*) 

  app load ["Haskellish", "Process", "FileSys", "Say", "Test", "Absyn", "AbsynPrint", 
            "VsrAbsyn", "parse", "AbsynLogic", "Normalise", "StaticScope" ];
	    
  open Say Test Haskellish List Absyn;
  infixr 0 $;

  use "PPSymbol.sml";

  val testname = "StaticScope.sml";

  (* test case is a pair: (filepath,result_spec).
     Each result spec is a list (one spec for each system in the file).
     Each list element is a list of triple. A triple for each generated
     transition (so should be more than source transitions if there are some
     dynamic scopes). Triple components: positivem conds, neg conds and 
     list of target groups bundled with scope *)
 
  (* WARNING: test engine is very fragile. In particular success depends on
     order in which transitions, targets, condition elements are processed. In
     theory these are all sets (unordered entities) so the order should be
     irrelevant. This has to be fixed if test engine breaks in future as side
     effect of changes in other modules *)

type tgtspec  = string * string list (* scope * tgt list *)
type transpec = string list * string list * tgtspec list (* pos * neg * tgts *)
type testspec = string * transpec list list (* filename * static transitions *)

val testcases : testspec list = [
      
      ("test/run/actions01.vsp", 
	[[(["topstate.*.first"], [], 
	[("topstate.*", ["topstate.*.second"])])]]),
      ("test/run/actions02.vsp",
	[[(["top.*.first"], [], 
	[("top.*", ["top.*.second"])])]]),
      ("test/run/actions03.vsp",
	[[(["topstate.*.Q1.Q11.P1"],[],
		[("topstate.*.Q1.Q11",["topstate.*.Q1.Q11.P2.*.A1"]),
		 ("topstate.*.Q1.Q12",["topstate.*.Q1.Q12.P4"])])]]),
      ("test/run/actions04.vsp",
	[[(["topstate.*.first"], [],
		[("topstate.*", ["topstate.*.second"])]),
	  (["topstate.*.second"], [],
		[("topstate.*", ["topstate.*.first"])]),
	  (["topstate.*.first"], [],
		[("topstate.*", ["topstate.*.second"])])
	]]),
      ("test/run/trans01.vsp",
	[[(["topstate.*.first"], [],
		[("topstate.*", ["topstate.*.first"])])]]),
      ("test/run/trans03.vsp",
	[[(["topstate.*.first"], [],
		[("topstate.*", ["topstate.*.first.*.second"])])]]),
      ("test/run/trans05.vsp",
	[[(["topstate.*.I.E.L.A.C"], ["topstate.*.I.F.Q"],
		[("topstate.*.I.E.L.A",["topstate.*.I.E.L.A.D"]),
		 ("topstate.*.I.F",["topstate.*.I.F.Q.*.H"])]),
	(["topstate.*.I.E.L.A.C", "topstate.*.I.F.Q"], [],
		[("topstate.*.I.E.L.A",["topstate.*.I.E.L.A.D"]),
		 ("topstate.*.I.F.Q.*",["topstate.*.I.F.Q.*.H"])])
	]]),
      ("test/run/trans07.vsp",
	[[ (["topstate.*.first"], [],
	    [("topstate.*", ["topstate.*.first.*.second.*.third"])])
	]]),
      ("test/run/trans08.vsp",
	[[ (["topstate.*.first"], [],
	    [("topstate.*", ["topstate.*.first.R1.second.R3.A",
			     "topstate.*.first.R1.second.R4.B",
			     "topstate.*.first.R2.third"])])
	]]),
      ("test/run/trans09.vsp",
	[[(["topstate.*.first.R1.second"],[],
	   [("topstate.*.first.R1", ["topstate.*.first.R1.second.*.A"]),
	    ("topstate.*.first.R2.third.*",
				    ["topstate.*.first.R2.third.*.B"])]),
	  (["topstate.*.first.R1.second"],[],
	   [("topstate.*.first.R1", ["topstate.*.first.R1.second.*.A"])]),
	  (["topstate.*.first.R2.third"],[],
	   [("topstate.*.first.R2", ["topstate.*.first.R2.third.*.B"])])
	]]),
	   
      ("test/run/trans10.vsp",
	[[ (["topstate.*.first"], [],
	    [("topstate.*", ["topstate.*.first.*.second.*.third"])])
	]]),
      ("test/run/history02.vsp",
	[[
	   (["topstate.*.first"], [], 
	    [("topstate.*", ["topstate.*.first"])]),
	   (["topstate.*.first"], [],
	    [("topstate.*", ["topstate.*.D"])]),
	   (["topstate.*.D"], [],
	    [("topstate.*", ["topstate.*.first"])]),
	   (["topstate.*.first.*.A"], [],
	    [("topstate.*.first.*", ["topstate.*.first.*.B"])]),
	   (["topstate.*.first.*.B"], [],
	    [("topstate.*.first.*", ["topstate.*.first.*.C"])]),
	   (["topstate.*.first.*.C"], [],
	    [("topstate.*.first.*", ["topstate.*.first.*.A"])]) ]])
      ];

type aor = unit;
type atr = unit;

  fun tgtagree ({stscope,tgts,stsrcs},ttgt :tgtspec) = 
	idmatch (Symbol.name $ stscope) (#1 ttgt) andalso
	idlstmatch (map (Symbol.name) (tgts)) (#2 ttgt)

  fun tranagree (tran :static tran,testspec : transpec ) =  
	idlstmatch (map (Symbol.name)  (#pos tran)) (#1 testspec) 
	andalso
	idlstmatch (map (Symbol.name) (#neg tran)) (#2 testspec)
	andalso 
	length (#annotation tran) = length (#3 testspec)
	andalso
	all2 tgtagree (#annotation tran,#3 testspec)
	

  fun testsys ({hierarchy=h,trans,evgroups,...} :(aor,atr) system, testspec :transpec list) =
	let 
	    val tr1 = List.concat $ map (Normalise.degroup evgroups) trans
			handle Fail _ => impossible "#~3"
	    val str = map (StaticScope.ss_tran h) tr1
			handle Fail _ => impossible "#~2"
	    val pairs = ListPair.zip (tr1,str) 
			handle Fail _ => impossible "#~1"
	    val _ = IfVHigh(fn ()=> (app (fn (t,st) => (print "Transition:\n";
				       AbsynPrint.prTranln t;
				       print "translated to:\n";
				       map StaticScope.prSTranln st;
				       println "")) pairs))
	in all2 tranagree (List.concat str,testspec)  end 
	
  fun testf ((fname,testspec) :testspec) :bool =
	let val model   = VsrAbsyn.translate (parse.parse fname NONE)
	    val _       = assert(fn () => length model = length testspec)
	    val bundled = ListPair.zip (model,testspec)
	in alltrue $ map testsys bundled end

  val _ = Test.init testname 
  val _ = bdd.init 10000 10000

  val _   = setverbosity VNorm
  val res = map (fn t => test testf t (" " ^ #1 t)) testcases;
  
  val _ = Test.finalise testname res  
  val _ = bdd.done();



	  

