(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/util/Err.sig,v
   Comment: Error handling for SCOPE
   Release: scope-0-15_final

     Copyright (C) 2001-2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   TODO: describe newly added facilities

   Err.sig,v 1.6 2004/05/05 21:29:05 wasowski Exp
*)

signature Err = sig

  type Location = Location.Location 
  
  datatype excty = Lexical      | Syntax   | Semant    | Assertion
                 | NotSupported | Internal | DebugInfo | Runtime
		 | WrongInput
  
  type exn_info = excty * Location * string option * string 
			(* type, pos, fname, msg *)
  exception ScopeExn of exn_info

  val release_mode: bool -> unit

  val impossible  : string   -> 'a
  val impossibleL : Location -> string -> 'a  (* pos, msg *)
  val intErr      : exn -> int -> 'a
  val lexicalErr  : Location -> string -> 'a  (* pos, msg *)
  val syntaxErr   : Location -> string option -> string -> 'a
  val semErr      :             string -> 'a
  val runtimeErr  : string -> 'a
  val wrongInput  : string -> 'a
  
  val warning     :             string -> unit
  val assert      : (unit -> bool) -> string -> bool    
  

  val addFile : string -> exn_info -> exn_info

  val finalize : (unit -> 'a) -> exn -> 'b

  val exnhandler : exn -> (unit -> 'a) -> 'a
  val exnhandler1 : exn -> 'a
  val exnhandler0 : exn -> unit

end
(*

  This structure provides a uniform error handling for translating program
  using several input files. The 

  [release m] switches release mode on if m is true. Otherwise swithes to
  debug mode. Release is defualt. Assertions are evaluated only in debug mode.

  [finalize f exn] Calls f and throws exn. To be used in exception handling.
  Resembles destructors in C++. Just put it in the handler and give it f to
  be function which performs loca clean up.
  
  Following functions are shortcuts for throwing exceptions. They never
  return an thus they never return any value. The type variable is there to 
  match any type needed in expressions (so the typechecker does not complain
  for exn not matching something else).

  [impossible ""] equivalent to raise InternalError "Internal Error."

  [impossible s ] equivalent to raise InternalError s.

  [intErr e s] handler for internal errors. exception e is the one caught.
  if e is Internal ScopeExn then it is propagated, but the message is
  extended with s. If e is ScopeExn but not internal it is simply propagted
  retaining the type, location and message. If e is not a ScopeExn it is
  changed to Internal ScopeExn with s being the message. s is the error
  number (should be an integer with no leading zeros). Deliberately use int
  to save space on quotes.

*)

